
package wpf.sample.simple.ejb;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.ejb.EJBException;
import javax.naming.InitialContext;

import com.ibm.websphere.management.AdminService;
import com.ibm.websphere.management.AdminServiceFactory;
import com.ibm.websphere.wpf.PartitionDefinition;
import com.ibm.websphere.wpf.PartitionManager;
import com.ibm.websphere.wpf.PartitionScope;

/**
 * Bean implementation class for Enterprise Bean: LetterPartitions
 */
public class LetterPartitionsBean implements javax.ejb.SessionBean {
	private javax.ejb.SessionContext mySessionCtx;
	private PartitionManager partitionMgr;
	private String scenario;
	private String servername;
	private Set activePartitions = Collections.synchronizedSet(new HashSet());
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
		try
		{
			// Get the partitionManager
			InitialContext ic = new InitialContext();
			partitionMgr = (PartitionManager)ic.lookup(PartitionManager.JNDI_NAME);
			// Determine which test scenario is being requested
			scenario = (String)ic.lookup("cell/persistent/WPFSampleScenarioID");
			// Determine the name of this server
			AdminService adminService = AdminServiceFactory.getAdminService();
			servername = adminService.getProcessName();
		}
		catch(Exception e)
		{
			System.out.println("********** setSessionContext() failed - exception = " + e.toString()); 
			throw new EJBException(e);
		}
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}

	/**
	 * Determine which test scenario is requested and call appropriate method to 
	 * initialize the partition definitions to be returned. 
	 */	
	public PartitionDefinition[] getPartitions()
	{
		PartitionDefinition[] partitions = null;
		if (scenario.equalsIgnoreCase("test1"))
			partitions = getTest1Partitions();
		else if (scenario.equalsIgnoreCase("test2"))
			partitions = getTest2Partitions();
		else if (scenario.equalsIgnoreCase("test3"))
			partitions = getTest3Partitions();
		else {
			String msg = "getPartitions() failed - invalid scenario testid = " + scenario;
			System.out.println("********** " + msg);
			throw new EJBException(msg);
		}
		return partitions;
	}

	/**
	 * Test Scenario 1 - Nine partitions named with numbers
	 */
	private PartitionDefinition[] getTest1Partitions()
	{
		String[] names = {"par1", "par2", "par3",
				  "par4", "par5", "par6",
				  "par7", "par8", "par9"};		
		PartitionDefinition[] partitions = new PartitionDefinition[names.length];
		for(int i = 0; i < names.length; ++i)
			partitions[i] = partitionMgr.createPartitionDefinition(names[i]);
		return partitions;
	}
	
	/**
	 * Test Scenario 2 - Six partitions named with letters
	 */
	private PartitionDefinition[] getTest2Partitions()
	{
		String[] names = {"parA", "parB", "parC",
			          "parD", "parE", "parF"};		
		PartitionDefinition[] partitions = new PartitionDefinition[names.length];
		for(int i = 0; i < names.length; ++i)
			partitions[i] = partitionMgr.createPartitionDefinition(names[i]);
		return partitions;
	}

	/**
	 * Test Scenario 3 - Six partitions named with letters in alpha classification
	 *                   Nine partitions named with numbers in numeric classification
	 */
	private PartitionDefinition[] getTest3Partitions()
	{
		String[] names = {"parA", "parB", "parC",
				  "parD", "parE", "parF",		
		                  "par1", "par2", "par3",
			          "par4", "par5", "par6",
                                  "par7", "par8", "par9"};	
        String[] classes = {"alpha", "alpha", "alpha",
			    "alpha", "alpha", "alpha",
			    "numeric", "numeric", "numeric",
                            "numeric", "numeric", "numeric", 
			    "numeric", "numeric", "numeric"};
		PartitionDefinition[] partitions = new PartitionDefinition[names.length];
		for(int i = 0; i < names.length; ++i)
			partitions[i] = partitionMgr.createPartitionDefinition(names[i], classes[i], PartitionScope.K_CLUSTER);
		return partitions;
	}


	/**
	 * This is called when a specific partition is assigned to this server process.
	 */
	public boolean partitionLoadEvent(String partitionName)
	{
		System.out.println(
             "********** partitionLoadEvent entered, partitionName="
              + partitionName);
		activePartitions.add(partitionName);
		return true;
	}

	/**
	 * This is called when previously assigned partition is withdrawn from this server.
	 */	
	public void partitionUnloadEvent(String partitionName)
	{
		System.out.println(
             "********** partitionUnloadEvent entered, partitionName=" 
             + partitionName);
		activePartitions.remove(partitionName);
	}

	/**
	 * This may be called periodically to verify that this server is functioning correctly if
	 * it was assigned a partition.
	 */	
	public boolean isPartitionAlive(String partitionName)
	{
		return(activePartitions.contains(partitionName));
	}
	
	/*
	 * This is part of the external/remote interface
	 * This method is called with the suffix of a partition name and
	 * returns a string indicating the status of that partition on this server
	 */
	 public String checkPartitionBySuffix(String suffix) {
	 	String calledFrom = "Suffix Check";
		return partitionStatusInfo("par" + suffix, calledFrom);
	 }

	/*
	 * This is part of the external/remote interface
	 * This method is called with the name of a partition and
	 * returns a string indicating the status of that partition on this server
	 */
	 public String checkPartitionByName(String partitionName) {
		String calledFrom = "Name Check  ";
		return partitionStatusInfo(partitionName, calledFrom);
	 }

	/*
	 * Called with a partition name and responds indicating if that partition
	 * is active on this server, and also provides the name of the server. 
	 */
	private String partitionStatusInfo(String partitionName, String calledBy)
	{
		if (isPartitionAlive(partitionName)) {
			System.out.println("********** " + calledBy + ": Name=" 
                               + partitionName + " is Active");
			return(calledBy + ": Name=" + partitionName 
			       + ",is active on Server=" + servername);
		} else {
			System.out.println("********** " + calledBy + ": Name=" 
			                   + partitionName + " called but NOT Active <<--- error");
			return(calledBy + ": Name=" + partitionName + ",is NOT active on Server=" 
			       + servername + ",,--- error");
		}
	}
}

